/*
 * Celestial Fusion
 * Software for digitized engine data acquisition and processing
 *
 * Author: James McCrae
 */

#include <allegro.h>
#include <string.h>
#include <stdio.h>
#include <math.h>

#include "cf.h"
#include "poll.h"
#include "parport.h"
#include "simulate.h"
#include "gui.h"

//Global variables (for screen size, etc)
//modifyable by command line arguments

int NUMINPUTSAMPLES=20;

int RESOLUTIONX=640;
int RESOLUTIONY=480;

float PULSETIMECONSIDERED=1.0;

float POLLDELAYSEC=0.05;

float UPDATESPERSEC=10.0;

int GRAPHXSIZEPIXELS;
int GRAPHYSIZEPIXELS;

double TRIPOMETER=0.0;

double CALC_H_APPROX=0.5;

int AVGSMOOTHSIZE=8;

/* Function declarations */
void alleg_init();
void cmdLineError();

/* Main Method */
int main(int argc, char ** argv) {
  
  bool dosimulate=false;

  //parse command line arguments
  for (int i=1;i<argc;i++) {
    if (strcasecmp(argv[i],"-S")==0)
      dosimulate=true;
    else if (strcasecmp(argv[i],"-R")==0) {
      if (argc<=i+2) cmdLineError();
      RESOLUTIONX=atoi(argv[i+1]);
      RESOLUTIONY=atoi(argv[i+2]);
      i+=2;
    }
    else if (strcasecmp(argv[i],"-T")==0) {
      if (argc<=i+1) cmdLineError();
      PULSETIMECONSIDERED=atof(argv[i+1]);
      i+=1;
    }
    else if (strcasecmp(argv[i],"-D")==0) {
      if (argc<=i+1) cmdLineError();
      POLLDELAYSEC=atof(argv[i+1]);
      i+=1;
    }
    else if (strcasecmp(argv[i],"-U")==0) {
      if (argc<=i+1) cmdLineError();
      UPDATESPERSEC=atof(argv[i+1]);
      i+=1;
    }
    else
      cmdLineError();
  }

  //dynamically set the number of input samples to the time for
  //important polls as well as the time between each poll
  NUMINPUTSAMPLES=(int)(ceil(PULSETIMECONSIDERED/POLLDELAYSEC));

  //set up environment
  alleg_init();

  //set up parallel port IO
  ParallelPortIO ppio;
  ppio.setSimulated(dosimulate);

  //set up simulator (used if needed)
  Simulator sim;

  //set up GUI
  GUI g;

  double lastpolltime=0.0;

  do {
    
    if (ppio.getSimulated()) {
      sim.update(&ppio);      
    }
    else {      
      ppio.addPoll(TACHBUSADDR,((double)clock())/CLOCKS_PER_SEC,ppio.pollSensor(TACHBUSADDR));
      ppio.addPoll(SPEEDBUSADDR,((double)clock())/CLOCKS_PER_SEC,ppio.pollSensor(SPEEDBUSADDR));
    }

    //update tripometer
    if (ppio.obtainLastPoll(SPEEDBUSADDR).getReading()<=15)
      TRIPOMETER+=(ppio.obtainLastPoll(SPEEDBUSADDR).getReading())/((double)SPEEDPULSESPERKM);     
    else
      textprintf(screen, font, RESOLUTIONX/2,RESOLUTIONY/4+20,15, "Error: %i", ppio.obtainLastPoll(SPEEDBUSADDR).getReading());
 
    g.incrementPoll();
    g.update(&ppio);        

    //printf("%f VS %f\n",((double)uclock())/UCLOCKS_PER_SEC,((double)clock())/CLOCKS_PER_SEC);
 
    //delay
    while (((double)(clock())/CLOCKS_PER_SEC)<lastpolltime+POLLDELAYSEC) { }
    lastpolltime=(double)(clock())/CLOCKS_PER_SEC;
    
  } while (true); 

}

/*
 * Sets up keyboard, timer and graphics mode.
 */
void alleg_init() {
  if (allegro_init()!=0) {
    exit(1);
  }  
  install_keyboard();

  set_color_depth(8);
  set_gfx_mode(GFX_AUTODETECT,RESOLUTIONX,RESOLUTIONY,0,0); 
  text_mode(-1);
}


void cmdLineError() {

  printf("Invalid usage of CF!\n");
  printf("Proper command line usage is of the form:\n");
  printf("  cf.exe [-S] [-R X Y] [-T t] [-D t] [-U n]\n");
  printf("-S\tSimulate sensor inputs (default is to not simulate)\n");
  printf("-R\tResolution of graphics, where X and Y are pixels (default 640 480)\n");
  printf("-T\tTime of polls taken t seconds ago considered (default is 1.0)\n");
  printf("-D\tDelay t seconds between polls (default is 0.05)\n");
  printf("-U\tUpdate GUI n times per second (default is 10)\n");

  exit(-1);

}


